module SkirentalFrontendApi
    class Booking
        include NoBrainer::Document
        include NoBrainer::Document::Timestamps

        before_save :sync_with_backend

        has_many :payments, class_name:SkirentalPayment::Payment ## TODO: Add autmatic delete + Add index

        field  :session_id,      		:type => String,                   :unique => true, :required => true,   :index => true
        field  :shop_id,         		:type => String,                                    :required => true,   :index => true
        field  :context,                :type => Hash,     :default => {}
        field  :booking_id, 			:type => String

        def voucher
            # TODO: Rethink whether this is good for the future. Future: If we want to update bookings or need
            # receipts, we need to store the booking transaction into the payment and have to handle multiple (old) payments
            confirmed_payment = self.payments.select { |payment|  payment.voucher_code != nil }.last
            if confirmed_payment.present?
                confirmed_payment.voucher_code
            else
                nil
            end
        end

        def transaction(use_tax = true)
            shop_settings = SkirentalApi::Client.settings(shop_id, "de")
            fail if context.nil?

            items = []
            start_date = Date.parse(context[:startDate]).strftime("%Y-%m-%d")
            duration = (Date.parse(context[:endDate]) - Date.parse(context[:startDate])).to_i
            currency = shop_settings[:money][:payment][:cur]
            price_total = 0
            tax_total = 0

            context[:customers].each do |customer|
                customer[:products].each do |product|
                    age = customer[:customerGroup][:ageFrom]

                    price_already_fetched = false

                    items.each do |item|
                        if item[:sku] == product[:id] && item[:age] == age
                            item[:quantity] += 1
                            price_total += item[:price].to_d
                            if use_tax
                                tax_total += item[:tax].to_d
                            end
                            product[:price] = item[:priceHash]
                            price_already_fetched = true
                            break
                        end
                    end

                    unless price_already_fetched
                        # 1. We expect that prices are independent to the lang.
                        # 2. We store product_id's therefore we do not access prices of packages.
                        price = SkirentalApi::Client.get_price_of_item(product[:id], start_date, duration, age, shop_id, "de", false)

                        fail if price.nil?
                        next if price[:grossPrice].blank? || price[:grossPrice] <= 0 || price[:netPrice].blank? || price[:netPrice] <= 0

                        item = {
                            :name => product[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : ""),
                            :description => product[:description],
                            :sku => product[:id],
                            :currency => currency,
                            :price => (use_tax ? price[:netPrice] : price[:grossPrice]),
                            :quantity => 1,
                            :age => age,
                            :priceHash => price
                        }

                        price_total += item[:price].to_d
                        if use_tax
                            item[:tax] = price[:vatValue]
                            tax_total += item[:tax].to_d
                        end

                        product[:price] = item[:priceHash]

                        items.push(item)
                    end
                end
            end

            items.each do |item|
                item.delete(:age)
                item.delete(:priceHash)
            end

            self.save

            {
                :item_list => {
                    :items => items
                },
                :amount =>  {
                    :total =>  (price_total + tax_total).to_f,
                    :currency =>  currency,
                    :details => {
                        :subtotal => price_total.to_f,
                        :tax => tax_total.to_f
                    }
                },
                :description =>  "Wir bedanken uns fuer Ihre Buchung!" # TODO: do we need a description???
            }
        end

        def sync_with_backend
            api_booking_id = SkirentalApi::Client.sync_booking_state(self)
            self.booking_id = api_booking_id if api_booking_id.present?
        end
    end
end
